#!/arch/bin/perl -w

###################################################################################################
#
# Name: indexandformat.pl
#
# This program is used to make index of fasta file to facilitate the searching and format the
# database for blast.
#
# Imput: name of input file including multiple fasta sequences and name of taxon
#
# Output: an index file (e.g., Danio_rerio.genome.new.index), a new data file (e.g., Danio_
#         rerio.genome.new.fas) and formatdb database. The orginal input file will be deleted to
#         save space.
#
# Author: Chenhong Li                                                                     
#         University of Nebraska - Lincoln                                                
#         NE 68588-0118 USA                                                               
#                                                                                         
# Created by:  June 3, 2010                                                              
#                                                                                         
# Last modified by:                                                                      
#
###################################################################################################

use strict;
use warnings;

use Getopt::Long;   # include the module for input


my $inputfile = "a"; #the input file name, the default is a
my $specieslist; #the name of species

my $opt = GetOptions( 'inputfile:s', \$inputfile,
                      'specieslist:s', \$specieslist); #set command line options

my @species = split(/\s/, $specieslist);

my ($OLD_FILE, $NEW_FILE, $INDEX_FILE);

foreach my $species (@species) {

    my ($old, $new, $index);
    
    $old = "../$species.db/species.fas";
    $new = "../$species.db/$species.genome.new.fas";
    $index = "../$species.db/$species.genome.new.index";

    `/arch/bin/formatdb -p F -i $old -n ../$species.db/$species.db`; #make a blast database
    
    open $OLD_FILE, "<$old" or die ("Cannot open $old for reading ($!)");
    open $NEW_FILE, ">$new" or die ("Cannot open $new for writing ($!)");
    open $INDEX_FILE, ">$index" or die ("Cannot open $index for writing ($!)");

    my ($id, $seq);
    my $file_pointer = 0;
    while (my $line = readline ($OLD_FILE)) {
        chomp $line;
        if ($line =~ /^>/) { # if we find >
            if ($id) { # if id exists we first need to write old information
        		print $INDEX_FILE "$id\t$file_pointer\n";
                $file_pointer += length(">$id\n$seq\n");
        	    print $NEW_FILE ">$id\n$seq\n";
            } 
        	# .. And then extract the new id and set sequence to empty
        	($id) = $line =~ /^>(\S+)/;
        	$seq = "";
        } else {
        		$seq .= $line;
        }
    }
    # We need to store the last sequence as well
    print $INDEX_FILE "$id\t$file_pointer\n";
    print $NEW_FILE ">$id\n$seq\n";

    close ($OLD_FILE) or die "Can't close the old file!!!";
    close ($NEW_FILE) or die "Can't close the new file!!!";
    close ($INDEX_FILE) or die "Can't close the new file!!!";



#    `rm $species.fasta`; #remove the original file

}
