#!/arch/bin/perl -w

###################################################################################################
#
# Name: makeindex.pl
#
# This program is used to make index of fasta file to facilitate the searching.
#
# Imput: name of input file (e.g., Danio_rerio.ZFISH7.50.dna.toplevel.fa) including multiple fasta 
#        sequences, prefix for name of output files (e.g., Danio_rerio.genome), input through @ARGV
#
# Output: an index file (e.g., Danio_rerio.genome.new.index) and a new data file (e.g., Danio_
#         rerio.genome.new.fas)
#
# Author: Chenhong Li                                                                     
#         University of Nebraska - Lincoln                                                
#         NE 68588-0118 USA                                                               
#                                                                                         
# Created by:  Nov. 5, 2008                                                              
#                                                                                         
# Last modified by:                                                                      
#
###################################################################################################

use strict;
use warnings;

my $old = shift(@ARGV); #get the file name from ARGV
my $outfile = shift(@ARGV); #get the keyword for output files
my $new = "$outfile.new.fas";
my $index = "$outfile.new.index";


my ($OLD_FILE, $NEW_FILE, $INDEX_FILE);
    
open $OLD_FILE, "<$old" or die ("Cannot open $old for reading ($!)");
open $NEW_FILE, ">$new" or die ("Cannot open $new for writing ($!)");
open $INDEX_FILE, ">$index" or die ("Cannot open $index for writing ($!)");

my ($id, $seq);
my $file_pointer = 0;
while (my $line = readline ($OLD_FILE)) {
chomp $line;
if ($line =~ /^>/) { # if we find >
    if ($id) { # if id exists we first need to write old information
		print $INDEX_FILE "$id\t$file_pointer\n";
        $file_pointer += length(">$id\n$seq\n");
	    print $NEW_FILE ">$id\n$seq\n";
    } 
	# .. And then extract the new id and set sequence to empty
	($id) = $line =~ /^>(.+)/;
	$seq = "";
	} else {
		$seq .= $line;
	}
}
# We need to store the last sequence as well
print $INDEX_FILE "$id\t$file_pointer\n";
print $NEW_FILE ">$id\n$seq\n";

close ($OLD_FILE) or die "Can't close the old file!!!";
close ($NEW_FILE) or die "Can't close the new file!!!";
